@@ -1,4 +1,4 @@
-from .differentiation import dxdt, methods
+from .differentiation import dxdt, smooth_x, methods
 from .dglobal import Spectral, Spline, TrendFiltered, Kalman
 from .dlocal import FiniteDifference, SavitzkyGolay
 
